@@ -17,6 +17,7 @@ from yafowil.common import (
 from yafowil.compound import compound_renderer
 
 
+@managedprops(*css_managed_props)
 def array_display_proxy_renderer(widget, data):
     input_attrs = {
         'type': 'hidden',
@@ -40,13 +41,21 @@ factory.register(
 factory.doc['blueprint']['array_display_proxy'] = UNSET
 
 
+ICON_CSS = {
+    'add': 'icon-plus-sign',
+    'remove': 'icon-minus-sign',
+    'up': 'icon-circle-arrow-up',
+    'down': 'icon-circle-arrow-down',
+}
+
 def actions_renderer(widget, data):
     tag = data.tag
     actions = list()
     for key in ['add', 'remove', 'up', 'down']:
         if widget.attrs.get(key):
             class_ = 'array_row_%s' % key
-            action = tag('a', '&#160;', href='#', class_=class_)
+            icon = tag('i', '&#160;', class_=ICON_CSS[key])
+            action = tag('a', icon, href='#', class_=class_)
             actions.append(action)
     kw = dict(class_='array_actions')
     return tag('div', *actions, **kw)
@@ -61,7 +70,9 @@ factory.doc['blueprint']['array_actions'] = UNSET
 
 
 def array_builder(widget, factory):
-    table = widget['table'] = factory('table', props={'structural': True})
+    table = widget['table'] = factory('table', props={
+                                      'structural': True,
+                                      'class': widget.attrs['table_class']})
     head = table['head'] = factory('thead', props={'structural': True})
     row = head['row'] = factory('tr', props={'structural': True})
     props = dict()
@@ -80,7 +91,7 @@ def array_builder(widget, factory):
     table['body'] = factory('tbody', props={'structural': True})
 
 
-@managedprops('array', *css_managed_props)
+@managedprops('add', 'remove', 'sort', 'static', 'table_class', *css_managed_props)
 def array_wrapper_renderer(widget, data):
     add_css = ''
     if not widget.attrs['static']:
@@ -93,6 +104,8 @@ def array_wrapper_renderer(widget, data):
         'id': cssid(widget, 'array'),
         'class': cssclasses(widget, data) + add_css,
     }
+    if not 'array' in kw['class'].split(' '):
+        kw['class'] = 'array %s' % kw['class']
     rendered = data.rendered
     return data.tag('div', rendered, **kw)
 
@@ -165,10 +178,8 @@ def create_array_entry(idx, widget, template, value):
         orgin = template[template.keys()[-1]]
         template = duplicate_recursiv(orgin)
         hook_array_template(child_widget, template)
-        # If array in array with compound. For some reason (propably
-        # compound_renderer), the compound template children get hooked to
-        # array widget. Remove them.
-        # XXX: figure out in more detail what happens.
+        # If array in array with compound. Compound template children get 
+        # hooked to array widget. Remove them.
         for key in child_widget.keys():
             if not key in ['table', CONTAINER]:
                 del child_widget[key]
@@ -241,9 +252,10 @@ def check_base_name_in_request(widget, request):
 
 
 def array_display_renderer(widget, data):
-    """XXX: either throw exception or use edit renderers.
+    """XXX
     """
-    return '<div>Array display</div>'
+    raise NotImplementedError(u"yafowil.widget.array: Display mode not "
+                              u"implemented yet")
 
 
 factory.register(
@@ -267,6 +279,11 @@ factory.defaults['array.error_class'] = 'error'
 
 factory.defaults['array.message_class'] = 'errormessage'
 
+factory.defaults['array.table_class'] = None
+factory.doc['props']['array.table_class'] = \
+"""CSS classes rendered on array table.
+"""
+
 factory.defaults['array.static'] = False
 factory.doc['props']['array.static'] = \
 """Array is immutable. No Array actions are rendered. If True, ``add``,
